/*
 * $Id: brctl.c,v 1.9 2000/01/21 11:13:04 root Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "libbridge.h"

int nametoindex(char *name)
{
	int ifindex;

	if ((ifindex = if_nametoindex(name)) != 0)
		return ifindex;

	fprintf(stderr, "interface %s doesn't exist\n", name);
	exit(-1);
}



void br_cmd_addbr(struct bridge *br, char *arg0, char *arg1)
{
	br_add_bridge(arg0);
}

void br_cmd_delbr(struct bridge *br, char *arg0, char *arg1)
{
	br_del_bridge(arg0);
}

void br_cmd_addif(struct bridge *br, char *arg0, char *arg1)
{
	br_add_interface(br, nametoindex(arg0));
}

void br_cmd_delif(struct bridge *br, char *arg0, char *arg1)
{
	br_del_interface(br, nametoindex(arg0));
}

void br_cmd_setageing(struct bridge *br, char *arg0, char *arg1)
{
	double ageing_time;

	sscanf(arg0, "%lf", &ageing_time);
	ageing_time *= HZ;
	br_set_ageing_time(br, ageing_time);
}

void br_cmd_setbridgeprio(struct bridge *br, char *arg0, char *arg1)
{
	int prio;

	sscanf(arg0, "%i", &prio);
	br_set_bridge_priority(br, prio);
}

void br_cmd_setfd(struct bridge *br, char *arg0, char *arg1)
{
	double fdelay;

	sscanf(arg0, "%lf", &fdelay);
	fdelay *= HZ;
	br_set_bridge_forward_delay(br, fdelay);
}

void br_cmd_setgcint(struct bridge *br, char *arg0, char *arg1)
{
	double gcint;

	sscanf(arg0, "%lf", &gcint);
	gcint *= HZ;
	br_set_gc_interval(br, gcint);
}

void br_cmd_sethello(struct bridge *br, char *arg0, char *arg1)
{
	double hello_time;

	sscanf(arg0, "%lf", &hello_time);
	hello_time *= HZ;
	br_set_bridge_hello_time(br, hello_time);
}

void br_cmd_setmaxage(struct bridge *br, char *arg0, char *arg1)
{
	double max_age;

	sscanf(arg0, "%lf", &max_age);
	max_age *= HZ;
	br_set_bridge_max_age(br, max_age);
}

void br_cmd_setpathcost(struct bridge *br, char *arg0, char *arg1)
{
	printf("not implemented yet!\n");
}

void br_cmd_setportprio(struct bridge *br, char *arg0, char *arg1)
{
	printf("not implemented yet!\n");
}

void br_cmd_stp(struct bridge *br, char *arg0, char *arg1)
{
	int stp;

	stp = 0;
	if (!strcmp(arg0, "on") || !strcmp(arg0, "yes") || !strcmp(arg0, "1"))
		stp = 1;

	br_set_stp_state(br, stp);
}

void br_cmd_show(struct bridge *br, char *arg0, char *arg1)
{
	if (arg0 != NULL && (br = br_find_bridge(arg0)) != NULL) {
		br_dump_info(br);
		return;
	}

	printf("bridge name\tbridge id\t\tSTP enabled\n");
	br = bridge_list;
	while (br != NULL) {
		printf("%s\t\t", br->ifname);
		br_dump_bridge_id((unsigned char *)&br->info.bridge_id);
		printf("\t%s\n", br->info.stp_enabled?"yes":"no");

		br = br->next;
	}
}

static int compare_fdbs(const void *_f0, const void *_f1)
{
	const struct fdb_entry *f0 = _f0;
	const struct fdb_entry *f1 = _f1;

	if (f0->port_no < f1->port_no)
		return -1;

	if (f0->port_no > f1->port_no)
		return 1;

	return memcmp(f0->mac_addr, f1->mac_addr, 6);
}

void br_cmd_showmacs(struct bridge *br, char *arg0, char *arg1)
{
	struct fdb_entry fdb[1024];
	int i;
	int num;

	num = br_get_fdb_list(br, fdb, 1024);
	qsort(fdb, num, sizeof(struct fdb_entry), compare_fdbs);

	printf("port\tmac addr\t\tis local?\tageing timer\n");
	for (i=0;i<num;i++) {
		struct fdb_entry *f;

		f = fdb + i;
		printf("%3i\t", f->port_no);
		printf("%.2x:%.2x:%.2x:%.2x:%.2x:%.2x\t",
		       f->mac_addr[0], f->mac_addr[1], f->mac_addr[2],
		       f->mac_addr[3], f->mac_addr[4], f->mac_addr[5]);
		printf("%s\t\t", f->is_local?"yes":"no");
		br_show_timer(f->ageing_timer_value);
		printf("\n");
	}
}








struct command
{
	int needs_bridge_argument;
	char *name;
	void (*func)(struct bridge *br, char *arg0, char *arg1);
} commands[] = {
	{0, "addbr", br_cmd_addbr},
	{1, "addif", br_cmd_addif},
	{0, "delbr", br_cmd_delbr},
	{1, "delif", br_cmd_delif},
	{1, "setageing", br_cmd_setageing},
	{1, "setbridgeprio", br_cmd_setbridgeprio},
	{1, "setfd", br_cmd_setfd},
	{1, "setgcint", br_cmd_setgcint},
	{1, "sethello", br_cmd_sethello},
	{1, "setmaxage", br_cmd_setmaxage},
	{1, "setpathcost", br_cmd_setpathcost},
	{1, "setportprio", br_cmd_setportprio},
	{0, "show", br_cmd_show},
	{1, "showmacs", br_cmd_showmacs},
	{1, "stp", br_cmd_stp},
};

struct command *lookup(char *cmd)
{
	int i;
	int numcommands;

	numcommands = sizeof(commands)/sizeof(commands[0]);

	for (i=0;i<numcommands;i++)
		if (!strcmp(cmd, commands[i].name))
			return &commands[i];

	return NULL;
}

char *help_message =
"syntax:\n"
"\taddbr\t\t<bridge>\t\tadd bridge\n"
"\taddif\t\t<bridge> <device>\tadd interface to bridge\n"
"\tdelbr\t\t<bridge>\t\tdelete bridge\n"
"\tdelif\t\t<bridge> <device>\tdelete interface from bridge\n"
"\tsetageing\t<bridge> <time>\t\tset ageing time\n"
"\tsetbrigeprio\t<bridge> <prio>\t\tset bridge priority\n"
"\tsetfd\t\t<bridge> <time>\t\tset bridge forward delay\n"
"\tsetgcint\t<bridge> <time>\t\tset garbage collection interval\n"
"\tsethello\t<bridge> <time>\t\tset hello time\n"
"\tsetmaxage\t<bridge> <time>\t\tset max message age\n"
"\tsetpathcost\t<bridge> <port> <cost>\tset path cost\n"
"\tsetportprio\t<bridge> <port> <prio>\tset port priority\n"
"\tshow\t\t\t\t\tshow a list of bridges\n"
"\tshow\t\t<bridge>\t\tshow bridge info\n"
"\tshowmacs\t<bridge>\t\tshow a list of mac addrs\n"
"\tstp\t\t<bridge> <state>\t{dis,en}able stp\n";

void help()
{
	fprintf(stderr, help_message);
}

int main(int argc, char *argv[])
{
	int argindex;
	struct bridge *br;
	struct command *cmd;

	br_init();

	if (argc < 2)
		goto help;

	if ((cmd = lookup(argv[1])) == NULL) {
		fprintf(stderr, "never heard of command [%s]\n", argv[1]);
		goto help;
	}

	argindex = 2;
	br = NULL;
	if (cmd->needs_bridge_argument) {
		if (argindex >= argc) {
			fprintf(stderr, "this option requires a bridge name as argument\n");
			return 1;
		}

		br = br_find_bridge(argv[argindex]);

		if (br == NULL) {
			fprintf(stderr, "bridge %s doesn't exist!\n", argv[argindex]);
			return 1;
		}

		argindex++;
	}

	cmd->func(br, argv[argindex], argv[argindex+1]);

	return 0;

help:
	help();
	return 1;
}
